#
# spec file
#
# Copyright (c) 2023 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


#
%define slurm_version %{nil}

%if "%slurm_version" != ""
%define _slurm_version _%{slurm_version}
%endif

%if 0%{!?sle_version:1} || 0%{?sle_version} >= 120200
%define have_munge 1
%define have_slurm 1
%define have_genders 1
%endif
%if 0%{?suse_version} >= 1550 || "%{?slurm_version}" != "18_08"
 %ifarch %ix86 %arm ppc s390
   %define have_slurm 0
 %endif
%endif

%if !0%{?have_slurm} && 0%{?_slurm_version:1}
ExclusiveArch:  do_not_build
%endif

%define pname pdsh
Name:           %{pname}%{?_slurm_version:_slurm%{?_slurm_version}}
BuildRequires:  dejagnu
BuildRequires:  openssh
BuildRequires:  readline-devel
%if 0%{?have_slurm}
BuildRequires:  slurm%{?_slurm_version}-devel
%endif
%if 0%{?have_munge}
BuildRequires:  munge-devel
%endif
BuildRequires:  pam-devel
Recommends:     mrsh
%if 0%{?have_genders}
BuildRequires:  genders-devel > 1.0
%endif
URL:            http://pdsh.googlecode.com/
Version:        2.34
Release:        12.1
Summary:        Parallel remote shell program
# git clone of https://code.google.com/p/pdsh/
License:        GPL-2.0-or-later
Group:          Productivity/Clustering/Computing
Source:         https://github.com/chaos/%{pname}/releases/download/%{pname}-%{version}/%{pname}-%{version}.tar.gz
Patch1:         slurm-add-C-features-constraint.patch
Patch2:         slurm-add-documentation-for-C.patch
Patch3:         hostlist-fix-use-of-strchr.patch
Patch4:         dshbak-fix-uninitialized-use-of-tag-on-empty-input.patch
Patch5:         Release-a-lock-that-is-no-longer-used-in-dsh.patch
Patch6:         fail-fast-on-ssh-errors-or-non-zero-return-code.patch
Patch7:         doc-fast-fail-update.patch
Patch8:         Hack-to-work-around-a-generic-type-name-breakage-introduced-by-latest-Slurm.patch
Patch9:         Add-call-to-slurm_init-this-makes-sure-the-config-options-are-set.patch

%description
Pdsh is a multithreaded remote shell client which executes commands on
multiple remote hosts in parallel.  Pdsh can use several different
remote shell services, including Kerberos IV and ssh.

%package -n     %{pname}-slurm%{?_slurm_version}
Summary:        SLURM plugin for pdsh
Group:          Productivity/Clustering/Computing
%{?_slurm_version:BuildRequires:  %pname}
Requires:       %pname = %{version}
Enhances:       slurm%{?_slurm_version}
%if 0%{?_slurm_version:1}
Provides:       %{pname}-slurm = %{version}
Conflicts:      %{pname}-slurm
%endif

%description -n %{pname}-slurm%{?_slurm_version}
Plugin for pdsh to determine nodes to run on by SLURM jobs or partitions.

%package genders
Summary:        Genders plugin for pdsh
Group:          Productivity/Clustering/Computing
Requires:       pdsh = %{version}
Conflicts:      pdsh-dshgroup
Conflicts:      pdsh-machines
Conflicts:      pdsh-netgroup

%description genders
Plugin for pdsh to determine nodes to run on by genders attributes.

%package machines
Summary:        Machines plugin for pdsh
Group:          Productivity/Clustering/Computing
Requires:       pdsh = %{version}
Conflicts:      pdsh-genders

%description machines
Plugin for pdsh to determine nodes to run on from machines file.

%package dshgroup
Summary:        Dsh plugin for pdsh
Group:          Productivity/Clustering/Computing
Requires:       pdsh = %{version}
Conflicts:      pdsh-genders
Conflicts:      pdsh-netgroup

%description dshgroup
Plugin for pdsh to determine nodes from dsh-style "group" files

%package netgroup
Summary:        Netgroup plugin for pdsh
Group:          Productivity/Clustering/Computing
Requires:       pdsh = %{version}
Conflicts:      pdsh-dshgroup
Conflicts:      pdsh-genders

%description netgroup
Plugin for pdsh to determine nodes to run on from netgroups.

%prep
%setup -q -n %{pname}-%{version}
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
%patch7 -p1
%patch8 -p1
%patch9 -p1

%build
export CFLAGS="%{optflags} -fno-strict-aliasing"
%configure \
%if 0%{?_slurm_version:1}
        --without-exec \
%else
	--with-readline \
	--with-machines=%{_sysconfdir}/pdsh/machines \
	--with-ssh \
	--with-dshgroups \
	--with-netgroup \
        --with-rcmd-rank-list="ssh %{?have_munge:mrsh} krb4 qsh mqsh exec xcpu" \
        --with-pam \
        %{?have_genders:--with-genders} \
        %{?have_munge:--with-mrsh} \
%endif
        %{?have_slurm:--with-slurm} \
	--without-rsh \
	--disable-static
%if 0%{!?make_build:1}
# accomodate < SLE-15
 %define make_build make %{?_smp_mflags}
%endif
%make_build

%install
%{?_slurm_version:cd src/modules}
%make_install
rm -f %buildroot/%{_libdir}/pdsh/*.la

%if !0%{?_slurm_version:1}
%check
make check

%files
%doc README DISCLAIMER.* README.* NEWS TODO
%license COPYING
%attr(755, root, root) %{_bindir}/pdsh
%attr(755, root, root) %{_bindir}/pdcp
%{_bindir}/dshbak
%{_bindir}/rpdcp
%{_mandir}/man1/pdsh.1.gz
%{_mandir}/man1/pdcp.1.gz
%{_mandir}/man1/dshbak.1.gz
%{_mandir}/man1/rpdcp.1.gz
%{_libdir}/pdsh
%{?have_genders:%exclude %{_libdir}/pdsh/genders.so}
%{?have_slurm:%exclude %{_libdir}/pdsh/slurm.so}
%exclude %{_libdir}/pdsh/machines.so
%exclude %{_libdir}/pdsh/dshgroup.so
%exclude %{_libdir}/pdsh/netgroup.so

%if 0%{?have_genders}
%files genders
%{_libdir}/pdsh/genders.so
%endif

%files machines
%{_libdir}/pdsh/machines.so

%files dshgroup
%{_libdir}/pdsh/dshgroup.so

%files netgroup
%{_libdir}/pdsh/netgroup.so
%endif # if _slurm_version

%if 0%{?have_slurm}
%files -n %{pname}-slurm%{?_slurm_version}
%{_libdir}/pdsh/slurm.so
%endif

%changelog
* Mon Jul  3 2023 Egbert Eich <eich@suse.com>
- Only call slurm_init() if Slurm > 21.02 (bsc#1212946).
* Thu Jun  8 2023 Egbert Eich <eich@suse.com>
- Preparing pdsh for Slurm 23.02.
* Mon Mar 13 2023 Egbert Eich <eich@suse.com>
- Fix slurm plugin: make sure slurm_init() is called before using
  the Slurm API. This has been the case since version 20.11 (bsc#1209216).
  Add-call-to-slurm_init-this-makes-sure-the-config-options-are-set.patch
* Thu Mar  2 2023 Egbert Eich <eich@suse.com>
- Hack-to-work-around-a-generic-type-name-breakage-introduced-by-latest-Slurm.patch
  Schedmd did it again! Slurm 23.02 broke the pdsh-internal
  List type by exposing it thru it's public API. The way to
  protect from it no longer works as types are actually used
  in the header.
  This is a hot fix on the pdsh side, that is hopefully sufficiently
  generic. (boo#1208846).
* Tue Jan  3 2023 Egbert Eich <eich@suse.com>
- Backport a number of features and fixes from the git master branch
  (boo#1206795):
  * Add '-C' option on Slrum plugin to restrict selected nodes to ones
    with the specified features present:
    slurm-add-C-features-constraint.patch
    slurm-add-documentation-for-C.patch
  * Add option '-k' to the ssh plugin to fail faster on connection failures:
    fail-fast-on-ssh-errors-or-non-zero-return-code.patch
    doc-fast-fail-update.patch
  * Fix use of strchr
    hostlist-fix-use-of-strchr.patch
  * Dshbak: Fix uninitialized use of $tag on empty input
    dshbak-fix-uninitialized-use-of-tag-on-empty-input.patch
  * Dsh: Release a lock that is no longer used in dsh()
    Release-a-lock-that-is-no-longer-used-in-dsh.patch
* Mon Oct 31 2022 Egbert Eich <eich@suse.com>
- Preparing pdsh for Slurm 22.05.
  * No later version of Slurm builds on 32 bit.
* Thu Feb 25 2021 Egbert Eich <eich@suse.com>
- Add an explicit Conflicts: when pdsh is rebuilt for a
  non-default version of Slurm.
* Thu Jan 28 2021 Egbert Eich <eich@suse.com>
- Preparing pdsh for Slurm 20.11.
* Wed Jan 27 2021 Egbert Eich <eich@suse.com>
- Simplify convoluted condition - no need for Leap 42.2 support.
* Sat Apr 18 2020 Egbert Eich <eich@suse.com>
- Bring QA to the package build: add %%%%check stage
* Fri Apr 10 2020 Egbert Eich <eich@suse.com>
- Since the build for the SLE-12 HPC Module got fixed, simplify
  spec file and remove legacy workarounds.
- Remove _multibuild file where not needed.
* Tue Mar 31 2020 Egbert Eich <eich@suse.com>
- Add better legacy support.
- Make sure to not ship duplicate files with builds for multiple
  SLURM versions.
* Fri Mar 27 2020 Egbert Eich <eich@suse.com>
- Define %%%%make_build for all products that do have it defined.
* Thu Mar 26 2020 Egbert Eich <eich@suse.com>
- Do not build for SLURM on 32bit platforms on Factory: SLURM 20.02
  doesn't support this.
* Thu Mar 26 2020 Ana Guerrero Lopez <aguerrero@suse.com>
- Spec cleanup
* Wed Mar 25 2020 Egbert Eich <eich@suse.com>
- Update to version pdsh-2.34 (2020-01-07) (bsc#1167704)
  * Fix for output corruption with no newlines (#114)
  * pipecmd: fix result check error handling (Dylan Simon)
  * slurm: workaround slurm export of internal List interfaces
  * readline: add application name as argv[0] (#112)
  * Fix errors from lgtm.com scan
- Remove obsolete patches:
  Change-typedef-List-to-typedef-xList.patch
  pdsh-rename-list-to-xlist.patch
* Tue Feb 18 2020 Egbert Eich <eich@suse.com>
- Add Change-typedef-List-to-typedef-xList.patch:
  * Change 'typedef List' to 'typedef xList'.
    Since SLURM exposes a type List in its public API without proper
    namespacing we need to change this type in pdsh.
    To remain compatible with older versions of SLURM whose type List
    was identical to pdsh's we need to rename not only the struct.
* Sat Dec  7 2019 Egbert Eich <eich@suse.com>
- Add support for an alternative SLURM version when building the
  slurm plugin.
- Use %%license macro
* Thu Mar 29 2018 jjolly@suse.com
- removed patch: pdsh-exclude-slurm-list-namespace.patch:
  Patch is not an acceptable work-around for upstream
- added patch: pdsh-rename-list-to-xlist.patch
  Renamed 'struct list' to 'struct xlist' to match upstream slurm
  change (bsc#1086736).
  Original upstream commit message:
    to avoid name collisions with c++ list types. gdb started showing
    the wrong type when the full lib was created.
* Wed Mar 28 2018 jjolly@suse.com
- added patch: pdsh-exclude-slurm-list-namespace.patch:
  Defined __list_datatypes_defined in list.h to prevent conflict
  with slurm 17.11 (bsc#1086736).
* Tue Feb 20 2018 eich@suse.com
- Since the different host-list plugins have conflicting
  options and a 'the first one wins' policy is not well
  defined, package all of them separately and make ones
  with identical options conflict (bsc#1081673).
* Fri Feb 16 2018 eich@suse.com
- Add genders support to pdsh (FATE#325288).
- Put SLURM plugin into separate package (FATE#325289).
* Wed Oct 11 2017 eich@suse.com
- Update to version 2.33 for SLE-12 (HPC) (FATE#324169).
* Mon Oct  9 2017 eich@suse.com
- Removed deprecated %%leap_version from spec file.
* Sat Oct  7 2017 jengelh@inai.de
- Remove --with-pic which is a no-op due to --disable-static.
- Replace old RPM constructs.
* Fri Oct  6 2017 eich@suse.com
- Update to 2.33:
  * Fix segfault and build issues on Mac OSX (#95)
  * Always pass RTLD_GLOBAL to dlopen(3) of modules. Fixes missing symbol
    errors from modules using libraries that also use dlopen() (e.g.
    nodeupdown, slurm)
  From 2.32:
  * Autotools update
  * Switch to dlopen(3)/dlsym(3) instead of using libltdl
  * Drop qshell, mqshell, rmsquery, nodeattr and sdr modules.
  * Fix issue 70: dshbak: handle hostname of "0"
  * Allow PDSH_CONNECT_TIMEOUT and PDSH_COMMAND_TIMEOUT environment
    variables (Erik Jacobson)
  * Fix some old URLs in documentation (Al Chu)
  * Avoid exporting POSIXLY_CORRECT to child processes (Dorian Krause)
  * Fix mcmd start offset bug in max bytes calculation (Egbert Eich)
- Removed:
  mcmd-Account-for-start-offset-when-providing-max-bytes-to-read.patch:
  Obsoleted by update.
* Fri Mar 10 2017 eich@suse.com
- Fix %%if clause in spec file.
* Thu Feb 16 2017 eich@suse.com
- mcmd-Account-for-start-offset-when-providing-max-bytes-to-read.patch
  Fix a write past the end of a buffer.
* Wed Feb 15 2017 eich@suse.com
- Add 'Recommends: mrsh':
  pdsh should be run using the mrsh protocol but can get by using
  ssh as well, thus adding as a recommended dependency.
* Mon Nov 21 2016 eich@suse.com
- Disable support for slurm and mrsh until it is in Factory.
* Tue Nov  8 2016 eich@suse.com
- Add support for: pam, exec, mrsh and slurm
  genders will follow later. We don't have support for this lib, yet.
* Sat Oct 22 2016 eich@suse.com
- Replace tarball with official tarball of 2.31. (Content identical).
  (Add download rpm).
- Add _service for download_files.
- Remove .la files from installation.
* Wed Oct 19 2016 eich@suse.com
- Importing to SLE-12-SP2 for FATE#321714.
* Mon Aug 10 2015 jkeil@suse.de
- Don't include the rsh module, because rsh is outdated, deprecated and upstream
  is dead.  Removing the dependency makes it possible to drop the rsh package.
* Thu Jan 30 2014 tabraham@suse.com
- update to version 2.31 (2013-11-07)
  - - updated to git tag pdsh-2.31 (rev:e1c8e71dd6a2)
  - - Fix issue 56: slurm: Allow mixed use of -P, -w and -j options.
  - - Fix issue 59: pdsh very slow when using a few thousand hosts and genders.
  - - testsuite: Expanded tests for genders module (Pythagoras Watson)
- Changes from pdsh-2.30 (2013-03-02)
  - - Fix issue 55: genders -X option removes more hosts than expected.
    (This was a generic fix for hostname matching, so it probably
    affected -x and other options as well.)
  - - testsuite: Add test for issue 55.
* Wed Oct  2 2013 david.bahi@emc.com
- update to version 2.29 (2013-02-12)
  - - Fix issue 42: --with-dshgroup sets DSHGROUP_PATH to "yes"
  - - Fix issue 53: Add -P option to target SLURM partitions (Michael Fenn)
  - - Fix issue 54: pdsh prints empty lines with -S
  - - pdcp: Add more levels of directory recursion (Pythagoras Watson)
* Mon Dec  3 2012 tabraham@suse.com
- update to version 2.28
  - - Fix issue 39: ssh forces use of -l<user> option
  - - Fix issue 40: -l%%u added to ssh args only if remote and local
    usernames differ
- update to version 2.27
  - - Fix issue 17: Allow dshgroup files to include other files
  - - Fix issue 33: dshbak breaks up host lists at zeropad boundaries,
    (e.g. 01-09,10-11, 010-099,100-101, etc.)
  - - Fix issue 34: dshgroup path override broken in ./configure
  - - Fix issue 36: pdsh truncates output lines at 8K
  - - dshgroup: Allow dshgroup search path to be overridden by DSHGROUP_PATH,
    a colon-separated list of directories to search. $HOME/.dsh/group
    is still always prepended to this path.
  - - Allow wcoll files (-w ^file and WCOLL=file) to include other files
    with a "#include FILE" syntax. If included files have no path, then
    a search path of the dirname of the included file ("." for stdin)
    is used. -- Fix issue 17: Allow dshgroup files to include other files
  - - Fix issue 33: dshbak breaks up host lists at zeropad boundaries,
    (e.g. 01-09,10-11, 010-099,100-101, etc.)
  - - Fix issue 34: dshgroup path override broken in ./configure
  - - Fix issue 36: pdsh truncates output lines at 8K
  - - dshgroup: Allow dshgroup search path to be overridden by DSHGROUP_PATH,
    a colon-separated list of directories to search. $HOME/.dsh/group
    is still always prepended to this path.
  - - Allow wcoll files (-w ^file and WCOLL=file) to include other files
    with a "#include FILE" syntax. If included files have no path, then
    a search path of the dirname of the included file ("." for stdin)
    is used.
  - - Fix some minor memory leaks and locking bugs reported by Coverity.
* Wed Nov  2 2011 cfarrell@suse.com
- license update: GPL-2.0+
  SDPX format (http://www.spdx.org/licenses)
* Tue Nov  1 2011 tabraham@novell.com
- update to version 2.26
  - - Fix issue 14: interactive mode broken with ssh
  - - Fix issue 19: missing commas in dshbak(1) header output
  - - Fix issue 20: compilation error in genders.c with non-GCC compilers
  - - Fix issue 23: compilation failure with --enable-static-modules
  - - Fix issue 24: don't arbitrarily limit usernames to 16 characters
  - - Fix issue 25: PDSH_SSH_ARGS should not require %%u and %%h
  - - Fix issue 26: document use of %%u and %%h in PDSH_SSH_ARGS
  - - Fix interactive mode with rcmd/exec.
  - - genders: do not look for genders_query(3) support at runtime,
    as this causes too many problems on some systems. Instead,
    use autoconf to include or exclude genders_query support from
    the genders module. (fixes Issue 1)
- update to version 2.25
  - - Fix pdcp breakage with ssh (bug introduced in pdsh-2.24).
    (Resolves issue 12: pdcp executes file instead of copying.)
  - - testsuite: Skip tests dependent on PDSH_MODULE_DIR when testsuite
    run as root (Resolves issue 13: testsuite broken when run as root)
  - - testsuite: Skip dshbak -d test to non-writable directory when
    directory permissions do not seem to apply (e.g. privileged user)
    (Possibly resolves issue 11: tests fail on Mac OSX under fink)
  - - testsuite: add simple ssh teststestsuite: add simple ssh tests
- update to version 2.24
  - - Resolve issue 7: Allow PDSH_REMOTE_PDCP_PATH to set default path
    to remote pdcp program (pdcp -e).
  - - Resolve issue 9: Fix use of PDSH_SSH_ARGS_APPEND.
  - - Resolve issue 10: dshbak: Support writing output to file-per-host.
    Adds new -d DIR and -f options to dshbak.
  - - genders: Allow relative paths to genders files with -F and
    PDSH_GENDERS_FILE.
  - - genders: Don't print an error if genders file is missing, unless
    a genders optin (-F, -a, -g, -i, -X) is explicitly used.
  - - genders: Allow -g to be used with other node selection options as
    a filter. For example: pdsh -w host[0-1024] -g attr1 ...
  - - ssh: Better preservation of remote command args in ssh module.
    Previous versions of pdsh would alwas collapse all args into
    a single argument passed to ssh: "cmd arg1 arg2 ..." With this
    fix the argv list will more closely match the form passed to pdsh
  - - Refactored large portions of dshbak, improve usage output,
    and update manpage
  - - Expanded testsuite.
- update to version 2.23
  - - Fix issue 4: List available but conflicting modules in -V and -L output
  - - Fix issue 5: dshbak -c doesn't properly coalesce hosts with different
    zero-padding in the numeric suffix.
  - - Added torque module for setting list of target hosts based on Torque/PBS
    jobs (Issue 2).
  - - Enhance syntax of -w and -x to allow preceeding arguments with:
    `-' - Exclude hosts instead of include with -w ( -w foo[1-9],-foo8)
    `^' - insert/exclude list of hosts from a file (^/path/to/file)
  `/' - filter hosts based on regex (/host.*0$/)
  - - Introduce new pdsh testsuite with many new tests. See tests/README
    for more information
* Thu Nov 18 2010 hvogel@novell.com
- update to version 2.22
  - - Sort modules by name before initialization so that modules
    initialize in a reproducible order.
  - - New option -M name,... forces load of misc modules by
    name, thus allowing users to select between conflicting
    modules.
  - - Fix parsing of hostlist expressions with multiple brackets
  - - Fix for coredump when no rcmd module is loaded.
  - - Fix duplicate error output from ssh module.
  - - Add -e option to pdcp to explicitly specify remote execution
    path.
* Wed May 12 2010 puzel@novell.com
- build with support for dshgroup and netgroup (bnc#477720)
* Fri Apr 11 2008 crrodriguez@suse.de
- fix build with glibc 28, ARG_MAX no longer defined
- update to version 2.16
- Rewrite of the ssh module using the same framework as the existing
  "exec" module.
- Ability to specify an ssh connect timeout.
- Small improvements for the dshbak script.
- Other minor fixes to the pdsh interface.
- See the NEWS file distributed with pdsh for further information.
* Thu Apr  3 2008 hvogel@suse.de
- update to 2.14
  * some build fixes
  * Enable "exec" rcmd module by default
  * Ignore blank and comment lines in WCOLL file
  * Add new "pipecmd" API for executing arbitrary commands
  * Add "-N" option to disable hostname
  * Block SIGCHLD in pdsh threads
  * Undocumented -K option to explicitly keep
    domain names in output labels
* Mon Apr 30 2007 ro@suse.de
- added rpdcp to filelist (binary and manpage)
* Wed Apr 25 2007 mskibbe@suse.de
- update to version 2.12 which includes:
  o pdsh-2.10.patch
  o fix build warnings
  o fix bugs
* Thu Aug 31 2006 mskibbe@suse.de
- update to version 2.10 which
  o allow rcmd modules to set rcmd-specific options
  o Add test for "," in host range. Increase version number
  o Add COPYING to docs. Remove -n from %%setup
  o Add support for command history in interactive mode when using
    readline (History file is placed in ~/.pdsh/history)
  o Change mini shell script so -S hack will work
* Thu Jul 13 2006 mskibbe@suse.de
- update to version 2.9 which
  o Remove unneeded .la and .a files in packages
  o Use io session file instead of stdout and stdin session files
  o Remove code which only loaded pdsh modules via *.la files
  o Add xcpu module support
  o Do not install pdsh/pdcp setuid root by default anymore
  o Update documentation
* Wed Jan 25 2006 mls@suse.de
- converted neededforbuild to BuildRequires
* Tue Jan 17 2006 hvogel@suse.de
- disable make check for now
* Fri Jan 13 2006 hvogel@suse.de
- update to version 2.8
* Wed Jun  8 2005 meissner@suse.de
- use RPM_OPT_FLAGS -fno-strict-aliasing
* Tue Jun  7 2005 hvogel@suse.de
- update to version 2.3
* Thu Feb 26 2004 hmacht@suse.de
- building as non-root
* Sun Jun 15 2003 nashif@suse.de
- Initial release (1.7-6)
